@@ -25,9 +25,9 @@
 
 #include <NetworkManagerQt/GsmSetting>
 
-GsmWidget::GsmWidget(const NetworkManager::Setting::Ptr &setting, QWidget* parent, Qt::WindowFlags f):
-    SettingWidget(setting, parent, f),
-    m_ui(new Ui::GsmWidget)
+GsmWidget::GsmWidget(const NetworkManager::Setting::Ptr &setting, QWidget *parent, Qt::WindowFlags f)
+    : SettingWidget(setting, parent, f)
+    , m_ui(new Ui::GsmWidget)
 {
     m_ui->setupUi(this);
 
@@ -35,6 +35,11 @@ GsmWidget::GsmWidget(const NetworkManager::Setting::Ptr &setting, QWidget* paren
     m_ui->labelNetworkId->setHidden(true);
     m_ui->networkId->setHidden(true);
 
+    m_ui->password->setPasswordOptionsEnabled(true);
+    m_ui->password->setPasswordOptionEnabled(PasswordField::NotRequired, true);
+    m_ui->pin->setPasswordOptionsEnabled(true);
+    m_ui->pin->setPasswordOptionEnabled(PasswordField::NotRequired, true);
+
     m_ui->type->addItem(i18nc("GSM network type", "Any"), NetworkManager::GsmSetting::Any);
     m_ui->type->addItem(i18n("3G Only (UMTS/HSPA)"), NetworkManager::GsmSetting::Only3G);
     m_ui->type->addItem(i18n("2G Only (GPRS/EDGE)"), NetworkManager::GsmSetting::GprsEdgeOnly);
@@ -43,21 +48,18 @@ GsmWidget::GsmWidget(const NetworkManager::Setting::Ptr &setting, QWidget* paren
     m_ui->type->addItem(i18n("Prefer 4G (LTE)"), NetworkManager::GsmSetting::Prefer4GLte);
     m_ui->type->addItem(i18n("4G Only (LTE)"), NetworkManager::GsmSetting::Only4GLte);
 
-    connect(m_ui->cbShowPasswords, &QCheckBox::toggled, this, &GsmWidget::showPasswords);
-    connect(m_ui->pinStorage, static_cast<void (KComboBox::*)(int)>(&KComboBox::currentIndexChanged), this, &GsmWidget::pinStorageChanged);
-    connect(m_ui->passwordStorage, static_cast<void (KComboBox::*)(int)>(&KComboBox::currentIndexChanged), this, &GsmWidget::passwordStorageChanged);
-
     connect(m_ui->apn, &KLineEdit::textChanged, this, &GsmWidget::slotWidgetChanged);
     connect(m_ui->password, &KLineEdit::textChanged, this, &GsmWidget::slotWidgetChanged);
+    connect(m_ui->password, &PasswordField::passwordOptionChanged, this, &GsmWidget::slotWidgetChanged);
     connect(m_ui->pin, &KLineEdit::textChanged, this, &GsmWidget::slotWidgetChanged);
+    connect(m_ui->pin, &PasswordField::passwordOptionChanged, this, &GsmWidget::slotWidgetChanged);
     connect(m_ui->username, &KLineEdit::textChanged, this, &GsmWidget::slotWidgetChanged);
-    connect(m_ui->passwordStorage, static_cast<void (KComboBox::*)(int)>(&KComboBox::currentIndexChanged), this, &GsmWidget::slotWidgetChanged);
-    connect(m_ui->pinStorage, static_cast<void (KComboBox::*)(int)>(&KComboBox::currentIndexChanged), this, &GsmWidget::slotWidgetChanged);
 
     KAcceleratorManager::manage(this);
 
-    if (setting)
+    if (setting && !setting->isNull()) {
         loadConfig(setting);
+    }
 }
 
 GsmWidget::~GsmWidget()
@@ -73,26 +75,29 @@ void GsmWidget::loadConfig(const NetworkManager::Setting::Ptr &setting)
     if (!number.isEmpty())
         m_ui->number->setText(number);
     m_ui->username->setText(gsmSetting->username());
-    if (gsmSetting->passwordFlags().testFlag(NetworkManager::Setting::None) ||
-        gsmSetting->passwordFlags().testFlag(NetworkManager::Setting::AgentOwned)) {
-        m_ui->passwordStorage->setCurrentIndex(SettingWidget::EnumPasswordStorageType::Store);
+    if (gsmSetting->passwordFlags().testFlag(NetworkManager::Setting::None)) {
+        m_ui->password->setPasswordOption(PasswordField::StoreForAllUsers);
+    } else if (gsmSetting->passwordFlags().testFlag(NetworkManager::Setting::AgentOwned)) {
+        m_ui->password->setPasswordOption(PasswordField::StoreForUser);
     } else if (gsmSetting->passwordFlags().testFlag(NetworkManager::Setting::NotSaved)) {
-        m_ui->passwordStorage->setCurrentIndex(SettingWidget::EnumPasswordStorageType::AlwaysAsk);
+        m_ui->password->setPasswordOption(PasswordField::AlwaysAsk);
     } else {
-        m_ui->passwordStorage->setCurrentIndex(SettingWidget::EnumPasswordStorageType::NotRequired);
+        m_ui->password->setPasswordOption(PasswordField::NotRequired);
     }
+
     m_ui->apn->setText(gsmSetting->apn());
     m_ui->networkId->setText(gsmSetting->networkId());
     if (gsmSetting->networkType() != NetworkManager::GsmSetting::Any)
         m_ui->type->setCurrentIndex(m_ui->type->findData(static_cast<int>(gsmSetting->networkType())));
     m_ui->roaming->setChecked(!gsmSetting->homeOnly());
-    if (gsmSetting->pinFlags() == NetworkManager::Setting::None ||
-        gsmSetting->pinFlags() == NetworkManager::Setting::AgentOwned) {
-        m_ui->pinStorage->setCurrentIndex(SettingWidget::EnumPasswordStorageType::Store);
-    } else if (gsmSetting->pinFlags() == NetworkManager::Setting::NotSaved) {
-        m_ui->pinStorage->setCurrentIndex(SettingWidget::EnumPasswordStorageType::AlwaysAsk);
+    if (gsmSetting->pinFlags().testFlag(NetworkManager::Setting::None)) {
+        m_ui->pin->setPasswordOption(PasswordField::StoreForAllUsers);
+    } else if (gsmSetting->pinFlags().testFlag(NetworkManager::Setting::AgentOwned)) {
+        m_ui->pin->setPasswordOption(PasswordField::StoreForUser);
+    } else if (gsmSetting->pinFlags().testFlag(NetworkManager::Setting::NotSaved)) {
+        m_ui->pin->setPasswordOption(PasswordField::AlwaysAsk);
     } else {
-        m_ui->pinStorage->setCurrentIndex(SettingWidget::EnumPasswordStorageType::NotRequired);
+        m_ui->pin->setPasswordOption(PasswordField::NotRequired);
     }
 
     loadSecrets(setting);
@@ -115,7 +120,7 @@ void GsmWidget::loadSecrets(const NetworkManager::Setting::Ptr &setting)
     }
 }
 
-QVariantMap GsmWidget::setting(bool agentOwned) const
+QVariantMap GsmWidget::setting() const
 {
     NetworkManager::GsmSetting gsmSetting;
     if (!m_ui->number->text().isEmpty())
@@ -124,11 +129,11 @@ QVariantMap GsmWidget::setting(bool agentOwned) const
         gsmSetting.setUsername(m_ui->username->text());
     if (!m_ui->password->text().isEmpty())
         gsmSetting.setPassword(m_ui->password->text());
-    if (m_ui->passwordStorage->currentIndex() == SettingWidget::EnumPasswordStorageType::Store) {
-        if (agentOwned) {
-            gsmSetting.setPasswordFlags(NetworkManager::Setting::AgentOwned);
-        }
-    } else if (m_ui->passwordStorage->currentIndex() == SettingWidget::EnumPasswordStorageType::AlwaysAsk) {
+    if (m_ui->password->passwordOption() == PasswordField::StoreForAllUsers) {
+        gsmSetting.setPasswordFlags(NetworkManager::Setting::None);
+    } else if (m_ui->password->passwordOption() == PasswordField::StoreForUser) {
+        gsmSetting.setPasswordFlags(NetworkManager::Setting::AgentOwned);
+    } else if (m_ui->password->passwordOption() == PasswordField::AlwaysAsk) {
         gsmSetting.setPasswordFlags(NetworkManager::Setting::NotSaved);
     } else {
         gsmSetting.setPasswordFlags(NetworkManager::Setting::NotRequired);
@@ -142,11 +147,11 @@ QVariantMap GsmWidget::setting(bool agentOwned) const
     gsmSetting.setHomeOnly(!m_ui->roaming->isChecked());
     if (!m_ui->pin->text().isEmpty())
         gsmSetting.setPin(m_ui->pin->text());
-    if (m_ui->pinStorage->currentIndex() == SettingWidget::EnumPasswordStorageType::Store) {
-        if (agentOwned) {
-            gsmSetting.setPinFlags(NetworkManager::Setting::AgentOwned);
-        }
-    } else if (m_ui->pinStorage->currentIndex() == SettingWidget::EnumPasswordStorageType::AlwaysAsk) {
+    if (m_ui->pin->passwordOption() == PasswordField::StoreForAllUsers) {
+        gsmSetting.setPinFlags(NetworkManager::Setting::None);
+    } else if (m_ui->pin->passwordOption() == PasswordField::StoreForUser) {
+        gsmSetting.setPinFlags(NetworkManager::Setting::AgentOwned);
+    } else if (m_ui->pin->passwordOption() == PasswordField::AlwaysAsk) {
         gsmSetting.setPinFlags(NetworkManager::Setting::NotSaved);
     } else {
         gsmSetting.setPinFlags(NetworkManager::Setting::NotRequired);
@@ -155,34 +160,20 @@ QVariantMap GsmWidget::setting(bool agentOwned) const
     return gsmSetting.toMap();
 }
 
-void GsmWidget::showPasswords(bool show)
-{
-    m_ui->password->setPasswordMode(!show);
-    m_ui->pin->setPasswordMode(!show);
-}
-
-void GsmWidget::pinStorageChanged(int index)
-{
-    m_ui->pin->setEnabled(index == SettingWidget::EnumPasswordStorageType::Store);
-}
-
-void GsmWidget::passwordStorageChanged(int index)
-{
-    m_ui->password->setEnabled(index == SettingWidget::EnumPasswordStorageType::Store);
-}
-
 bool GsmWidget::isValid() const
 {
     bool passwordUserValid = true;
     bool pinValid = true;
 
-    if (m_ui->passwordStorage->currentIndex() == SettingWidget::EnumPasswordStorageType::Store) {
+    if (m_ui->password->passwordOption() == PasswordField::StoreForUser ||
+        m_ui->password->passwordOption() == PasswordField::StoreForAllUsers) {
         passwordUserValid = !m_ui->username->text().isEmpty() && !m_ui->password->text().isEmpty();
-    } else if (m_ui->passwordStorage->currentIndex() == SettingWidget::EnumPasswordStorageType::AlwaysAsk) {
+    } else if (m_ui->password->passwordOption() == PasswordField::AlwaysAsk) {
         passwordUserValid = !m_ui->username->text().isEmpty();
     }
 
-    if (m_ui->pinStorage->currentIndex() == SettingWidget::EnumPasswordStorageType::Store) {
+    if (m_ui->pin->passwordOption() == PasswordField::StoreForUser ||
+        m_ui->pin->passwordOption() == PasswordField::StoreForAllUsers) {
         pinValid = !m_ui->pin->text().isEmpty();
     }
 
